/*
 * This file is part of essential (http://essential.craftforge.net).
 *
 *     Essential is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Lesser Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Essential is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2011 Christian Bick.
 */


package net.craftforge.essential.context.servlet;

import net.craftforge.essential.controller.Response;
import net.craftforge.essential.controller.constants.HttpStatusCode;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Map;

/**
 * A servlet implementation of a controller response.
 *
 * @author Christian Bick
 * @since 20.02.2011
 */
public class HttpServletResponseImpl implements Response {

    private HttpStatusCode httpStatusCode;
    private Map<String, String[]> headers;
    private HttpServletResponse response;

    /**
     * Initializes the response from a HTTP servlet response.
     *
     * @param response The HTTP servlet response
     */
    public HttpServletResponseImpl(HttpServletResponse response) {
        this.response = response;
    }

    /**
     * {@inheritDoc}
     */
    public void sendHeaders(HttpStatusCode httpStatusCode, Map<String, String[]> headers) {
        this.httpStatusCode = httpStatusCode;
        this.headers = headers;
        response.setStatus(httpStatusCode.getCode());
        for (String headerName : headers.keySet()) {
            for (String headerValue : headers.get(headerName)) {
                response.addHeader(headerName, headerValue);
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    public HttpStatusCode getStatusCode() {
        return httpStatusCode;
    }

    /**
     * {@inheritDoc}
     */
    public Map<String, String[]> getHeaders() {
        return headers;
    }

    /**
     * {@inheritDoc}
     */
    public OutputStream getBodyOutputStream() {
        try {
            return response.getOutputStream();
        } catch (IOException e) {
            throw new RuntimeException("Unable to resolve response body output stream", e);
        }
    }
}
