/*
 * This file is part of essential (http://essential.craftforge.net).
 *
 *     Essential is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Lesser Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Essential is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2011 Christian Bick.
 */

package net.craftforge.essential.testsuite;

import net.craftforge.essential.context.servlet.EssentialHttpServlet;
import net.craftforge.essential.testsuite.NetworkClientTest;
import net.craftforge.essential.testsuite.TestValues;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.nio.SelectChannelConnector;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.junit.After;
import org.junit.Before;

import javax.servlet.http.HttpServlet;
import java.net.URL;

/**
 * @author Christian Bick
 * @since 01.09.11
 */
public abstract class HttpServletTest extends NetworkClientTest {

    private HttpServlet servlet;
    private String baseUrl;
    private String packagePath;
    private Server server;

    public HttpServletTest () {
        this(new EssentialHttpServlet());
    }

    public HttpServletTest (HttpServlet servlet) {
        this(TestValues.DEFAULT_BASE_URL, TestValues.DEFAULT_PACKAGE_PATH, servlet);
    }

    public HttpServletTest (String baseUrl, String packagePath, HttpServlet servlet) {
        super(baseUrl);
        this.baseUrl = baseUrl;
        this.packagePath = packagePath;
        this.servlet = servlet;
    }

    /**
     * {@inheritDoc}
     * Sets up the test suite.
     *
     * @throws Exception
     */
    @Before
    public void setUp() throws Exception {
        server = new Server();
        SelectChannelConnector connector = new SelectChannelConnector();
        URL url = new URL(baseUrl);
        connector.setHost(url.getHost());
        connector.setPort(url.getPort());
        server.addConnector(connector);

        ServletContextHandler contextHandler = new ServletContextHandler(server, url.getPath());
        ServletHolder holder = new ServletHolder(servlet);
        holder.setInitParameter("package", packagePath);
        contextHandler.addServlet(holder, "/*");
        server.start();
    }

    /**
     * Stops the Jetty servlet container using a custom servlet which invokes an Essential controller.
     *
     * @throws Exception if an unexpected exception occurs
     */
    @After
    public void tearDown() throws Exception {
        server.stop();
    }
}
